# from naas_drivers.gsheet import Gsheet

# SPREADSHEET_ID = "10NHJf8Ny1x9dfg30jLpYPmg9o1UBp5ylwWxKt5EtBH0"
# SHEET_ID = "TSLA"


def test_gsheet():
    # gs = Gsheet()
    # df = gs(spreadsheet_id=SPREADSHEET_ID, sheet_id=SHEET_ID)
    # assert not df.empty
    assert True
